// Code modifié pour le projet Jump Analyser par Jérôme Humbert

// Ce code a été initialement écrit par : 

///////////////////////////////
// © 2020 Dmitriy Kharutskiy //
///////////////////////////////

// BMA180 triple axis accelerometer sample code //
// http://www.geeetech.com/wiki/index.php/BMA180_Triple_Axis_Accelerometer_Breakout //

#include <Wire.h> 
#include <ArduinoBLE.h>
#include <Arduino_LSM9DS1.h>

#define ACCELEROMETER_ADDRESS 0x6B // Address of the accelerometer
#define ACCELEROMETER_STATUS_REG 0x17 // (Page 49 of the datasheet for the LSM9DS1, STATUS_REG)
#define DATA_X 0x28 // (Page 56, OUT_X_L_XL)
#define DATA_Y 0x2A // (Page 56, OUT_Y_L_XL)
#define DATA_Z 0x2C // (Page 56, OUT_Z_L_XL)

//************************************************************
// General settings
//************************************************************

#define ACCELEROMETER_SETTINGS_ADDRESS 0x20 // (Page 51 of the datasheet for the LSM9DS1, CTRL_REG6_XL)
//#define ACCELEROMETER_SETTINGS B10100000 // B[101]00000 - 476 Hz, B110[00]000 - 2 G (Page 51-52 of the datasheet for the LSM9DS1, CTRL_REG6_XL)
//#define ACCELEROMETER_SETTINGS B10110000 // B[101]10000 - 476 Hz, B110[10]000 - 4 G
//#define ACCELEROMETER_SETTINGS B10111000 // B[101]01000 - 476 Hz, B110[11]000 - 8 G
#define ACCELEROMETER_SETTINGS B10101000 // B[101]01000 - 476 Hz, B110[01]000 - 16 G
//#define ACCELEROMETER_SETTINGS B10011000 // B[100]11000 - 238 Hz, B110[11]000 - 8 G
//#define ACCELEROMETER_SETTINGS B01111000 // B[011]11000 - 119 Hz, B110[11]000 - 8 G
//#define ACCELEROMETER_SETTINGS B11011000 // B[110]11000 - 952 Hz, B110[11]000 - 8 G // DOES NOT WORK. The test shows 476 Hz instead of 952 Hz.

#define ACCELEROMETER_RESET_ADDRESS 0x22 // (Page 53, CTRL_REG8)
#define ACCELEROMETER_RESET B00000101 // (Page 53, CTRL_REG8)

int16_t offx = 0;
int16_t offy = 0;
int16_t offz = 0;

const int bufferSize = 3; // For Bluetooth 4.2 and Bluetooth 5.0 https://punchthrough.com/maximizing-ble-throughput-part-3-data-length-extension-dle-2/
//const int bufferSize = 20; // For Bluetooth 4.0 https://punchthrough.com/maximizing-ble-throughput-part-3-data-length-extension-dle-2/
BLEService accelerometerService("31d98611-49de-4683-97ae-132bd1491ae5");
BLECharacteristic accelerometerDataChar("31d98612-49de-4683-97ae-132bd1491ae5", BLERead | BLENotify, bufferSize, true);

int dataLengthAfterMarker = bufferSize * 3;

//************************************************************


signed char ByteArr[bufferSize];
int i = 0;
int ii = 0;
int16_t AccelValue1 = 0;
int16_t AccelValue2 = 0;
int16_t AccelValue3 = 0;
int16_t LastAccelValue1 = 0;
int16_t LastAccelValue2 = 0;
int16_t LastAccelValue3 = 0;
uint16_t unsignedAccelValue = 0;
int16_t dValue1 = 0;
int16_t dValue2 = 0;
int16_t dValue3 = 0;
long previousMicros = 0;


void setup() 
{
  // put your setup code here, to run once:

  Serial.begin(9600);

  pinMode(LED_BUILTIN, OUTPUT); // initialize the built-in LED pin to indicate when a central is connected

  
  Wire1.begin(); 
  delay(50);

 
  // Initializing sensors
  initAccelerometer(); 

  initBLE();

  
  delay(500); // Pauses the program for the amount of time (in miliseconds) specified as parameter.
}

void loop() 
{
  // put your main code here, to run repeatedly:


  // wait for a BLE central
  BLEDevice central = BLE.central();
  
  
  // if a central is connected to the peripheral:
  if (central)
  {
    // turn on the LED to indicate the connection:
    digitalWrite(LED_BUILTIN, HIGH);


    // while the central is connected:
    while (central.connected()) 
    { 
      if (IMU.accelerationAvailable())
      { 
        /*
        // Sample rate testing
        long currentMicros = micros();
          Serial.println(currentMicros - previousMicros);
          previousMicros = micros();
          */
        int x;
        int y;
        int z;
        float x2, y2, z2;
        IMU.readAcceleration(x2, y2, z2);
        x = x2*100;
        y = y2*100;
        z = z2*100;
        Serial.print(x);
        Serial.print("\t");
        Serial.print(y);
        Serial.print("\t");
        Serial.println(z);

        ByteArr[ii] = x%256;
        ii++;
        ByteArr[ii] = y%256;
        ii++;
        ByteArr[ii] = z%256;
        ii++;
        sendBuffer();
        
      }
    }
    
    // when the central disconnects, turn off the LED:
    digitalWrite(LED_BUILTIN, LOW);
  }
}

//************************************************************
// Functions
//************************************************************

// Initializing sensors
void initAccelerometer() 
{ 
  byte temp[1];
  byte temp1;

  // RESET
  writeTo(ACCELEROMETER_ADDRESS, ACCELEROMETER_RESET_ADDRESS, 0x05);

  // ACCELEROMETER SETTING: acceleration range, band width
  writeTo(ACCELEROMETER_ADDRESS, ACCELEROMETER_SETTINGS_ADDRESS, ACCELEROMETER_SETTINGS);

  delay(10);
}

// Initializing BLE
void initBLE() 
{ 
  // begin initialization
  if (!BLE.begin()) 
  {   
    while (1)
    {
      Serial.println("starting BLE failed!");
      delay(1000);
    }
  }

  BLE.setLocalName("Vibration_Analysis_App");
  BLE.setAdvertisedService(accelerometerService); // add the service UUID
  accelerometerService.addCharacteristic(accelerometerDataChar); // add the battery level characteristic
  BLE.addService(accelerometerService); // Add the battery service

  // start advertising
  BLE.advertise();
}

// Writes val to address register on ACC
void writeTo(int DEVICE, byte address, byte val) 
{
  Wire1.beginTransmission(DEVICE); // start transmission to ACC
  Wire1.write(address); // send register address
  Wire1.write(val); // send value to write
  Wire1.endTransmission(); // end transmission
}

// Reads num bytes starting from address register in to buff array
void readFrom(int DEVICE, byte address, int num, byte buff[])
{
  Wire1.beginTransmission(DEVICE); // start transmission to ACC
  Wire1.write(address); // send register address
  Wire1.endTransmission(); // end transmission
 
  Wire1.beginTransmission(DEVICE); // start transmission to ACC
  Wire1.requestFrom(DEVICE, num); // request num byte from ACC
 
  int n = 0;
  while(Wire1.available()) // ACC may abnormal
  {
    buff[n] = Wire1.read(); // receive a byte
    n++;
  }
  Wire1.endTransmission(); // end transmission
}


void sendBuffer()
{
    long beforeMicros = micros();
    
    accelerometerDataChar.writeValue(ByteArr, ii);  
    
    long afterMicros = micros();    
    //Serial.println(afterMicros - beforeMicros); // Testing delay when sending data
        
    ii = 0;
    
    if (i >= dataLengthAfterMarker) i = 0;
    delay(100);
}